#!/bin/bash -f

mkdir -p ${BANYAN_DIR:-/var/log/banyan}
./stop-net.sh

if [[ $EUID -ne 0 ]]; then
	echo "Run with sudo"
	exit 1
fi

function check_error {
	err=$?
	if [[ ${err} -ne 0 ]]; then
		echo "$1 Error: ${err}"
		exit 1
	fi
}

# Download and decrypt kernel module
./netting-kms.sh
check_error "netting-kms"

echo running $0 at $(date)
echo creating ipsets
ipset create banyan hash:ip,port
check_error "ipset"

PATH=/opt/banyan/local/sbin:$PATH

echo installing ebtables rules

MASK=0x40000000
# MARK is the packet mark set by the proxy on outgoing traffic
MARK=0x40000000/0x40000000
# MARK_EB is the packet mark on proxied traffic sent to containers/pods attached to the bridge
MARK_EB=0x20000000/0x60000000

sysctl -q -w net.ipv4.conf.all.route_localnet=1
check_error "sysctl route_localnet"

function do_ebtables {
	ebtables "$@"
	check_error "ebtables $*"
}

do_ebtables -t nat -N BANYAN
do_ebtables -t nat -I PREROUTING -j BANYAN
do_ebtables -t filter -N BANYANFILTER
do_ebtables -t filter -I OUTPUT -j BANYANFILTER
do_ebtables -A BANYANFILTER --mark ${MARK_EB} -j ACCEPT


# policy routing rules check if packet should be routed to a local socket that may be using a non-local IP address.
echo installing routing rules
ip rule add fwmark ${MARK} lookup 100
check_error "ip rule"
ip route add local 0.0.0.0/0 dev lo table 100
check_error "ip route"

exit 0

# For Reference:
# netagent does the following:
# do_iptables -t raw -N BANYAN
# do_iptables -t raw -I OUTPUT -p tcp -j BANYAN
# do_iptables -t raw -I PREROUTING -p tcp -j BANYAN
# do_iptables -t raw -A BANYAN -m mark --mark ${MARK} -m comment --comment "packets sent by the proxy use conntrack zone 1" -j CT --zone 1
# do_iptables -t raw -A BANYAN -m mark --mark ${MARK} -j RETURN
# do_iptables -t raw -A BANYAN -p tcp -m set --match-set banyan src,src -m comment --comment "packets received from server use conntrack zone 1" -j CT --zone 1

# both proxies spoof the client IP address when they set up an outgoing connection to the server.
# Match incoming packets that are returning to the spoofed address, and mark them for policy routing.
# Reply packets from the proxy that is addressed to the same client IP address will not match,
# because traffic generated by a local process doesn't traverse the PREROUTING chain 
# do_iptables -t mangle -N BANYAN
# do_iptables -t mangle -A BANYAN -m mark --mark ${MARK} -j CONNMARK --save-mark --mask ${MASK} -m comment --comment "First save proxy connection packet mark to connmark"
# do_iptables -t mangle -A BANYAN -m mark --mark ${MARK} -j MARK --set-mark ${MARK_EB} -m comment --comment "Second delete packet mark for proxy connection"
# do_iptables -t mangle -A BANYAN -m conntrack --ctdir REPLY -m connmark --mark ${MARK} -j CONNMARK --restore-mark --mask ${MASK} -m comment --comment "Finally restore packet mark only for reply traffic of proxy connections"
# 
# do_iptables -t mangle -N BANYANPOST
# do_iptables -t mangle -A BANYANPOST -p tcp --tcp-flags SYN,ACK SYN -m connmark --mark ${MARK} -m comment --comment "add entry to identify traffic returning to the proxy" -j SET --add-set banyan dst,dst
# 
# do_iptables -t mangle -I PREROUTING -p tcp -j BANYAN -m comment --comment "Intercept outgoing traffic from containers and incoming traffic from the network"
# do_iptables -t mangle -I OUTPUT -p tcp -j BANYAN -m comment --comment "Intercept outgoing traffic from local processes"
# do_iptables -t mangle -I POSTROUTING -p tcp -j BANYANPOST
